@@ -1,7 +1,7 @@
 package Smart::Args::TypeTiny;
 use strict;
 use warnings;
-our $VERSION = "0.02";
+our $VERSION = "0.10";
 use Carp ();
 use PadWalker qw/var_name/;
 
@@ -38,6 +38,7 @@ sub args {
     my $args = (@DB::args == 1 && ref $DB::args[0] eq 'HASH')
             ?    $DB::args[0]  # must be hash
             : +{ @DB::args };  # must be key-value list
+    my $kv = {};
 
     # args my $var => RULE
     #         ~~~~    ~~~~
@@ -50,7 +51,7 @@ sub args {
 
         # with rule (my $foo => RULE, ...)
         if (defined $_[$i+1]) {
-            $_[$i] = check_rule($_[$i+1], $args->{$name}, exists $args->{$name}, $name);
+            $_[$i] = $kv->{$name} = check_rule($_[$i+1], $args->{$name}, exists $args->{$name}, $name);
             delete $args->{$name};
             $i++;
         }
@@ -59,13 +60,15 @@ sub args {
             unless (exists $args->{$name}) {
                 Carp::confess("Required parameter '$name' not passed");
             }
-            $_[$i] = delete $args->{$name};
+            $_[$i] = $kv->{$name} = delete $args->{$name};
         }
     }
 
     for my $name (sort keys %$args) {
         Carp::confess("Unexpected parameter '$name' passed");
     }
+
+    return $kv;
 }
 
 sub args_pos {
@@ -90,6 +93,7 @@ sub args_pos {
     }
 
     my $args = [@DB::args];
+    my $kv = {};
 
     # args my $var => RULE
     #         ~~~~    ~~~~
@@ -102,7 +106,7 @@ sub args_pos {
 
         # with rule (my $foo => RULE, ...)
         if (defined $_[$i+1]) {
-            $_[$i] = check_rule($_[$i+1], $args->[0], @$args > 0, $name);
+            $_[$i] = $kv->{$name} = check_rule($_[$i+1], $args->[0], @$args > 0, $name);
             shift @$args;
             $i++;
         }
@@ -111,13 +115,15 @@ sub args_pos {
             unless (@$args > 0) {
                 Carp::confess("Required parameter '$name' not passed");
             }
-            $_[$i] = shift @$args;
+            $_[$i] = $kv->{$name} = shift @$args;
         }
     }
 
     if (@$args) {
         Carp::confess('Too many parameters passed');
     }
+
+    return $kv;
 }
 
 1;
@@ -132,18 +138,17 @@ Smart::Args::TypeTiny - We are smart, smart for you
 =head1 SYNOPSIS
 
     use Smart::Args::TypeTiny;
-    use Types::Standard -all;
 
     sub func2 {
-        args my $p => Int,
-             my $q => {isa => Int, optional => 1},
+        args my $p => 'Int',
+             my $q => {isa => 'Int', optional => 1},
              ;
     }
     func2(p => 3, q => 4); # p => 3, q => 4
     func2(p => 3);         # p => 3, q => undef
 
     sub func3 {
-        args my $p => {isa => Int, default => 3};
+        args my $p => {isa => 'Int', default => 3};
     }
     func3(p => 4); # p => 4
     func3();       # p => 3
@@ -179,28 +184,131 @@ Smart::Args::TypeTiny - We are smart, smart for you
 
 Smart::Args::TypeTiny provides L<Smart::Args>-like argument validator using L<Type::Tiny>.
 
-=head1 IMCOMPATIBLE CHANGES WITH Smart::Args
+=head2 IMCOMPATIBLE CHANGES WITH Smart::Args
 
-=head2 ISA TAKES Type::Tiny TYPE OBJECT OR INSTANCE CLASS NAME
+=over 4
 
-This code is expected C<$p> as InstanceOf['Int'], you should specify L<Type::Tiny>'s type constraint.
-
-    use Types::Standard -all;
+=item Unexpected parameters will be a fatal error
 
     sub foo {
-        args my $p => 'Int', # :( InstanceOf['Int']
-             my $q => Int,   # :) Int
-             my $r => 'Foo', # :) InstanceOf['Foo']
+        args my $x => 'Str';
     }
 
-=head2 DEFAULT PARAMETER CAN TAKE CODEREF AS LAZY VALUE
+    sub bar {
+        args_pos my $x => 'Str';
+    }
+
+    foo(x => 'a', y => 'b'); # fatal: Unexpected parameter 'y' passed
+    bar('a', 'b');           # fatal: Too many parameters passed
+
+=item Optional allows to pass undef to parameter
 
     sub foo {
-        args my $p => {isa => 'Foo', default => create_foo},         # :( create_foo is called every time even if $p is passed
-             my $q => {isa => 'Foo', default => sub { create_foo }}, # :) create_foo is called only when $p is not passed
+        args my $p => 'Int';
+    }
+
+    foo();           # $p = undef
+    foo(p => 1);     # $p = 1
+    foo(p => undef); # $p = undef
+
+=item Default parameter can take coderef as lazy value
+
+    sub foo {
+        args my $p => {isa => 'Foo', default => create_foo},         # calls every time even if $p is passed
+             my $q => {isa => 'Foo', default => sub { create_foo }}, # calls only when $p is not passed
              ;
     }
 
+=back
+
+=head1 FUNCTIONS
+
+=over 4
+
+=item my $args = args my $var[, $rule], ...;
+
+    sub foo {
+        args my $int   => 'Int',
+             my $foo   => 'Foo',
+             my $bar   => {isa => 'Bar',  default  => sub { Bar->new }},
+             my $baz   => {isa => 'Baz',  optional => 1},
+             my $bool  => {isa => 'Bool', default  => 0},
+             ;
+
+        ...
+    }
+
+    foo(int => 1, foo => Foo->new, bool => 1);
+
+Check parameters and fills them into lexical variables. All the parameters are mandatory by default.
+The hashref of all parameters is returned.
+
+C<$rule> can be any one of type name, L<Type::Tiny>'s type constraint object, or hashref have these parameters:
+
+=over 4
+
+=item isa C<Str|Object>
+
+Type name or L<Type::Tiny>'s type constraint.
+
+=over 4
+
+=item Types::Standard
+
+    args my $int => {isa => 'Int'};
+
+=item Mouse::Util::TypeConstraints
+
+    use Mouse::Util::TypeConstraints;
+
+    subtype 'PositiveInt',
+        as 'Int',
+        where { $_ > 0 },
+        message { 'Must be greater than zero' };
+
+    args my $positive_int => {isa => 'PositiveInt'};
+
+=item class name
+
+    {
+        package Foo;
+        ...
+    }
+
+    args my $foo => {isa => 'Foo'};
+
+=back
+
+=item does C<Str|Object>
+
+Role name or L<Type::Tiny>'s type constraint object.
+
+=item optional C<Bool>
+
+The parameter doesn't need to be passed when L<optional> is true.
+
+=item default C<Any|CodeRef>
+
+Default value for the parameter.
+
+=back
+
+=item my $args = args_pos my $var[, $rule], ...;
+
+    sub bar {
+        args_pos my $x => 'Str',
+                 my $p => 'Int',
+                 ;
+
+        ...
+    }
+
+    bar('abc', 123);
+
+Same as C<args> except take arguments instead of parameters.
+
+=back
+
 =head1 TIPS
 
 =head2 SKIP TYPE CHECK
@@ -217,9 +325,9 @@ For optimization calling subroutine in runtime type check, you can overwrite C<c
 
 =head1 SEE ALSO
 
-L<Smart::Args>, L<Params::Validate>, L<Params::ValidationCompiler>
+L<Smart::Args>, L<Data::Validator>, L<Params::Validate>, L<Params::ValidationCompiler>
 
-L<Type::Tiny>, L<Types::Standard>
+L<Type::Tiny>, L<Types::Standard>, L<Mouse::Util::TypeConstraints>
 
 =head1 LICENSE
 
